==> tpb-0.2.1.tar.gz: tpb-0.2.1/tpb.c <==
/* tpb - program to use the IBM ThinkPad(tm) special keys
 * Copyright (C) 2002 Markus Braun <markus.braun@krawel.de>
 *
 * This file is part of tpb.
 *
 * tpb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * tpb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with tpb; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* includes {{{ */
#include <fcntl.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <wait.h>

#include "config.h"
#include "tpb.h"

#ifdef HAVE_LIBXOSD
#include <xosd.h>
#endif /* HAVE_LIBXOSD */
/* }}} */

/* RCS version string for later identification using rcs ident {{{*/
#ifndef lint
static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.4 2002/07/03 15:17:02 mbr Exp $";
#endif
/* }}} */

int main(int argc, char **argv)/*{{{*/
{
	int verbose=0;
#ifdef HAVE_LIBXOSD
	int osd=0;
	xosd *osdptr=NULL;
#endif /* HAVE_LIBXOSD */
	int polltime=POLLTIME;
  char *tpbcmd=NULL;
	t_tpb tpbs,ltpbs;

	/* Ignore SIGCHLD, so I don't have to wait for terminated childs (zombies) */
	signal(SIGCHLD,SIG_IGN);

	/* parse options {{{ */
	while (1) {
		int option_index = 0;
		char *endptr;
		int c;
		static struct option long_options[] = {
			{"help",     0, NULL, 'h'},
#ifdef HAVE_LIBXOSD
			{"osd",      0, NULL, 'o'},
#endif /* HAVE_LIBXOSD */
			{"poll",     1, NULL, 'p'},
			{"thinkpad", 1, NULL, 't'},
			{"verbose",  0, NULL, 'v'},
			{NULL,       0, NULL,   0}
		};

#ifdef HAVE_LIBXOSD
		c = getopt_long (argc, argv, "hop:t:v", long_options, &option_index);
#else
		c = getopt_long (argc, argv, "hp:t:v", long_options, &option_index);
#endif /* HAVE_LIBXOSD */
		if (c == -1)
			break;

		switch (c) {
			case 'h':
				print_usage(argv[0]);
				_exit(1);
				break;
#ifdef HAVE_LIBXOSD
			case 'o':
				osd=1;
				break;
#endif /* HAVE_LIBXOSD */
			case 'p':
				polltime=strtol(optarg, &endptr, 10);
				if(strlen(endptr)>0) {
					printf("Illegal polltime \"%s\"\n",optarg);
					_exit(1);
				}
				break;
			case 't':
				tpbcmd=optarg;
				break;
			case 'v':
				verbose=1;
				break;
			default:
				print_usage(argv[0]);
				_exit(1);
		}
	} /* }}}*/

#ifdef HAVE_LIBXOSD
	/* initialize osd */
	if(osd) osdptr=xosd_init(OSDFONT, OSDCOLOR, OSDTIMEOUT, OSDPOS, OSDOFFSET, 0);
#endif /* HAVE_LIBXOSD */

	/* to initialize struct */
	if(get_state(&tpbs)) {
		_exit(1);
	}

	while(1) {
		/* save last state and get new one */
		memcpy(&ltpbs,&tpbs,sizeof(t_tpb));
		if(get_state(&tpbs)) {
			_exit(1);
		}

		/* determine the state of the Thinkpad button  {{{ */
		if((tpbs.tpb != ltpbs.tpb) && (tpbs.ht == ltpbs.ht)) {
			if (verbose) puts("ThinkPad button pressed");
			if (tpbcmd!=NULL) {
				if(fork_app(tpbcmd)) {
					_exit(0);
				}
			}
		} /* }}} */

		/* determine the state of ThinkLight {{{ */
		if(tpbs.kt != ltpbs.kt) {
			if (verbose) printf("ThinkLight is %s\n", tpbs.kt ? "on" : "off");
		} /* }}} */

		/* determine the state of display  {{{ */
		if(tpbs.ds != ltpbs.ds) {
			if (verbose) printf("Display changed: LCD %s, CRT %s\n", tpbs.ds & 0x01 ? "on" : "off", tpbs.ds & 0x02 ? "on" : "off");
		} /* }}} */

		/* determine the state of the brightness buttons {{{ */
		if(tpbs.bl != ltpbs.bl) {
			if (verbose) printf("Brightness changed: Level %d\n", tpbs.bl);
		}
#ifdef HAVE_LIBXOSD
		if(tpbs.bt != ltpbs.bt) {
			if (osdptr) {
				xosd_display(osdptr, 0, XOSD_string, "Brightness");
				xosd_display(osdptr, 1, XOSD_percentage, tpbs.bl*100/7);
			}
		}
#endif /* HAVE_LIBXOSD */ /* }}} */

		/* determine the state of the volume buttons {{{ */
		if(tpbs.vl != ltpbs.vl) {
			if (verbose) printf("Volume changed: Level %d\n", tpbs.vl);
		}
#ifdef HAVE_LIBXOSD
    /* show volume every time a volume button is pressed and not mute */
		if(tpbs.vt != ltpbs.vt && !tpbs.mt) {
			if (osdptr) {
				xosd_display(osdptr, 0, XOSD_string, "Volume");
				xosd_display(osdptr, 1, XOSD_percentage, tpbs.vl*100/14);
			}
		}
#endif /* HAVE_LIBXOSD */ /* }}} */

		/* determine the state of the mute button {{{ */
		if(tpbs.mt != ltpbs.mt) {
			if (verbose) printf("%s\n", tpbs.mt ? "Mute" : "Unmute");
		}
#ifdef HAVE_LIBXOSD
		if(tpbs.mt != ltpbs.mt || (tpbs.vt != ltpbs.vt && ltpbs.mt)) {
			if (osdptr) {
				if(tpbs.mt) {
					xosd_display(osdptr, 0, XOSD_string, "Mute");
					xosd_display(osdptr, 1, XOSD_percentage, 0);
				}
				else {
					xosd_display(osdptr, 0, XOSD_string, "Unmute");
					xosd_display(osdptr, 1, XOSD_percentage, tpbs.vl*100/14);
				}
			}
		}
#endif /* HAVE_LIBXOSD */ /* }}} */

		/* determine the state of power {{{ */
		if(tpbs.ac != ltpbs.ac) {
			if (verbose) printf("Power line changed: %s\n", tpbs.ac ? "AC connected" : "AC disconnected");
#ifdef HAVE_LIBXOSD
			if (osdptr) {
					xosd_display(osdptr, 0, XOSD_string, tpbs.ac ? "AC connected" : "AC disconnected");
					xosd_display(osdptr, 1, XOSD_string, "");
			}
#endif /* HAVE_LIBXOSD */
		} /* }}} */

		/* determine power management mode ac {{{ */
		if(tpbs.pma != ltpbs.pma) {
			char *msg;
			switch(tpbs.pma) {
				case 0x4: msg="PM ac high"; break;
				case 0x2: msg="PM ac auto"; break;
				case 0x1: msg="PM ac manual"; break;
				default: msg="PM ac unknown"; break;
			}
			if (verbose) printf("Power management mode ac changed: %s\n", msg);
#ifdef HAVE_LIBXOSD
			if (osdptr) {
					xosd_display(osdptr, 0, XOSD_string, msg);
					xosd_display(osdptr, 1, XOSD_string, "");
			}
#endif /* HAVE_LIBXOSD */
		} /* }}} */

		/* determine power management mode battery {{{ */
		if(tpbs.pmb != ltpbs.pmb) {
			char *msg;
			switch(tpbs.pmb) {
				case 0x4: msg="PM battery high"; break;
				case 0x2: msg="PM battery auto"; break;
				case 0x1: msg="PM battery manual"; break;
				default: msg="unknown"; break;
			}
			if (verbose) printf("Power management mode battery changed: %s\n", msg);
#ifdef HAVE_LIBXOSD
			if (osdptr) {
					xosd_display(osdptr, 0, XOSD_string, msg);
					xosd_display(osdptr, 1, XOSD_string, "");
			}
#endif /* HAVE_LIBXOSD */
		} /* }}} */

		/* sleep for polltime */
		usleep(polltime);
	}

	_exit(0); /* never reached */
}/*}}}*/

void print_usage(const char *prog)/*{{{*/
{
	printf("%s 0.1 (C) 2002 Markus Braun [%s]\n\n",prog,RCSid);
	printf("Usage: %s options\n\n",prog);
	printf("options:\n");
	printf("   -h, --help           display this help\n");
#ifdef HAVE_LIBXOSD
	printf("   -o, --osd            show on-screen display for volume, mute and brightness\n");
#endif /* HAVE_LIBXOSD */
	printf("   -p, --poll=DELAY     set delay between polls in microseconds [%d]\n",POLLTIME);
	printf("   -t, --thinkpad=CMD   string with command and options that should be executed\n");
	printf("                        when ThinkPad button is pressed [none]\n");
	printf("   -v, --verbose        print information about pressed keys\n");
	return;
}/*}}}*/

int get_state(t_tpb *tpbs)/*{{{*/
{
	int  fdsc;
	char buffer[114];

	/* open nvram for reading */
	/* must use open/close because seek is not supported by nvram */
	if((fdsc=open(NVRAMDEV, O_RDONLY|O_NONBLOCK)) == -1) {
		perror("Unable to open device");
		return(-1);
	}

	/* read nvram */
	if(read(fdsc,buffer,sizeof(buffer)) != sizeof(buffer)) {
		perror("Unable to read from device");
		return(-1);
	}

	close(fdsc);

	tpbs->tpb=(buffer[0x57] & 0x08) >> 3;
	tpbs->kt=(buffer[0x58] & 0x10) >> 4;
	tpbs->ht=(buffer[0x58] & 0x01);
	tpbs->ds=(buffer[0x59] & 0x03);
	tpbs->bl=(buffer[0x5E] & 0x07);
	tpbs->bt=(buffer[0x5E] & 0x20) >> 5;
	tpbs->vl=(buffer[0x60] & 0x0f);
	tpbs->vt=(buffer[0x60] & 0x80) >> 7;
	tpbs->mt=(buffer[0x60] & 0x40) >> 6;
	tpbs->ac=(buffer[0x57] & 0x10) >> 4;
	tpbs->pma=(buffer[0x39] & 0x07);
	tpbs->pmb=(buffer[0x39] & 0x38) >> 3;

	return(0);
}/*}}}*/

int fork_app(char * cmd) { /* {{{ */
	//char *args[4];
	//args[0] = "sh";
	//args[1] = "-c";
	//args[2] = cmd;
	//args[3] = 0;

	switch(fork()) {
		case -1:
			return(-1);
			break;
		case 0:
			{
				setsid(); /* children should not be killed if tpb ends */
				//execv("/bin/sh", args);
				system(cmd);
				_exit(0);
				return(127);
			}
			break;
		default:
			return(0);
			break;
	}
	return(-1); /* never reached */
} /* }}} */

/* vim:set sw=2:set ts=2: */
/* vim600:set fen:set fdm=marker:set fdl=0: */

==> tpb-0.2.2.tar.gz: tpb-0.2.2/tpb.c <==
/* tpb - program to use the IBM ThinkPad(tm) special keys
 * Copyright (C) 2002 Markus Braun <markus.braun@krawel.de>
 *
 * This file is part of tpb.
 *
 * tpb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * tpb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with tpb; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* includes {{{ */
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <wait.h>
#include "config.h"

#ifdef HAVE_LIBXOSD
#include <xosd.h>
#endif /* HAVE_LIBXOSD */

#include "cfg.h"
#include "tpb.h"
/* }}} */

/* RCS version string for later identification using rcs ident {{{*/
#ifndef lint
static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.6 2002/07/09 12:05:18 mbr Exp $";
#endif
/* }}} */

extern config cfg;

int main(int argc, char **argv)/*{{{*/
{
#ifdef HAVE_LIBXOSD
	xosd *osdptr=NULL;
#endif /* HAVE_LIBXOSD */
	t_tpb tpbs,ltpbs;
	char *home;
	char *cfg_file;
	FILE *fdsc;

	/* Ignore SIGCHLD, so I don't have to wait for terminated childs (zombies) */
	signal(SIGCHLD,SIG_IGN);

	/* initialize config */
	init_cfg();

	/* read user config file */
	if((home=getenv("HOME")) != NULL) {
		cfg_file = (char *)malloc(strlen(home)+strlen("/.tpbrc")+1);
		strcpy(cfg_file,home);
		strcat(cfg_file,"/.tpbrc");
		if((fdsc=fopen(cfg_file, "r")) != NULL) {
			fclose(fdsc);
			if(parse_cfg_file(cfg_file)) {
				_exit(1);
			}
		}
		free(cfg_file);
	}

	/* parse options */
	parse_option(argc, argv);

#ifdef HAVE_LIBXOSD
	/* initialize osd */
	if(cfg.osd) osdptr=xosd_init(cfg.osdfont, cfg.osdcolor, cfg.osdtimeout, cfg.osdpos, cfg.osdoffset, 0);
#endif /* HAVE_LIBXOSD */

	/* to initialize struct */
	if(get_state(&tpbs)) {
		_exit(1);
	}

	while(1) {
		/* save last state and get new one */
		memcpy(&ltpbs,&tpbs,sizeof(t_tpb));
		if(get_state(&tpbs)) {
			_exit(1);
		}

		/* determine the state of the Thinkpad button  {{{ */
		if((tpbs.tpb != ltpbs.tpb) && (tpbs.ht == ltpbs.ht)) {
			if (cfg.verbose) puts("ThinkPad button pressed");
			if (cfg.tpbcmd!=NULL) {
				if(fork_app(cfg.tpbcmd)) {
					_exit(0);
				}
			}
		} /* }}} */

		/* determine the state of ThinkLight {{{ */
		if(tpbs.kt != ltpbs.kt) {
			if (cfg.verbose) printf("ThinkLight is %s\n", tpbs.kt ? "on" : "off");
		} /* }}} */

		/* determine the state of display  {{{ */
		if(tpbs.ds != ltpbs.ds) {
			if (cfg.verbose) printf("Display changed: LCD %s, CRT %s\n", tpbs.ds & 0x01 ? "on" : "off", tpbs.ds & 0x02 ? "on" : "off");
		} /* }}} */

		/* determine the state of the brightness buttons {{{ */
		if(tpbs.bl != ltpbs.bl) {
			if (cfg.verbose) printf("Brightness changed: Level %d\n", tpbs.bl);
		}
#ifdef HAVE_LIBXOSD
		if(tpbs.bt != ltpbs.bt) {
			if (osdptr) {
				xosd_display(osdptr, 0, XOSD_string, "Brightness");
				xosd_display(osdptr, 1, XOSD_percentage, tpbs.bl*100/7);
			}
		}
#endif /* HAVE_LIBXOSD */ /* }}} */

		/* determine the state of the volume buttons {{{ */
		if(tpbs.vl != ltpbs.vl) {
			if (cfg.verbose) printf("Volume changed: Level %d\n", tpbs.vl);
		}
#ifdef HAVE_LIBXOSD
    /* show volume every time a volume button is pressed and not mute */
		if(tpbs.vt != ltpbs.vt && !tpbs.mt) {
			if (osdptr) {
				xosd_display(osdptr, 0, XOSD_string, "Volume");
				xosd_display(osdptr, 1, XOSD_percentage, tpbs.vl*100/14);
			}
		}
#endif /* HAVE_LIBXOSD */ /* }}} */

		/* determine the state of the mute button {{{ */
		if(tpbs.mt != ltpbs.mt) {
			if (cfg.verbose) printf("%s\n", tpbs.mt ? "Mute" : "Unmute");
		}
#ifdef HAVE_LIBXOSD
		if(tpbs.mt != ltpbs.mt || (tpbs.vt != ltpbs.vt && ltpbs.mt)) {
			if (osdptr) {
				if(tpbs.mt) {
					xosd_display(osdptr, 0, XOSD_string, "Mute");
					xosd_display(osdptr, 1, XOSD_percentage, 0);
				}
				else {
					xosd_display(osdptr, 0, XOSD_string, "Unmute");
					xosd_display(osdptr, 1, XOSD_percentage, tpbs.vl*100/14);
				}
			}
		}
#endif /* HAVE_LIBXOSD */ /* }}} */

		/* determine the state of power {{{ */
		if(tpbs.ac != ltpbs.ac) {
			if (cfg.verbose) printf("Power line changed: %s\n", tpbs.ac ? "AC connected" : "AC disconnected");
#ifdef HAVE_LIBXOSD
			if (osdptr) {
					xosd_display(osdptr, 0, XOSD_string, tpbs.ac ? "AC connected" : "AC disconnected");
					xosd_display(osdptr, 1, XOSD_string, "");
			}
#endif /* HAVE_LIBXOSD */
		} /* }}} */

		/* determine power management mode ac {{{ */
		if(tpbs.pma != ltpbs.pma) {
			char *msg;
			switch(tpbs.pma) {
				case 0x4: msg="PM ac high"; break;
				case 0x2: msg="PM ac auto"; break;
				case 0x1: msg="PM ac manual"; break;
				default: msg="PM ac unknown"; break;
			}
			if (cfg.verbose) printf("Power management mode ac changed: %s\n", msg);
#ifdef HAVE_LIBXOSD
			if (osdptr) {
					xosd_display(osdptr, 0, XOSD_string, msg);
					xosd_display(osdptr, 1, XOSD_string, "");
			}
#endif /* HAVE_LIBXOSD */
		} /* }}} */

		/* determine power management mode battery {{{ */
		if(tpbs.pmb != ltpbs.pmb) {
			char *msg;
			switch(tpbs.pmb) {
				case 0x4: msg="PM battery high"; break;
				case 0x2: msg="PM battery auto"; break;
				case 0x1: msg="PM battery manual"; break;
				default: msg="unknown"; break;
			}
			if (cfg.verbose) printf("Power management mode battery changed: %s\n", msg);
#ifdef HAVE_LIBXOSD
			if (osdptr) {
					xosd_display(osdptr, 0, XOSD_string, msg);
					xosd_display(osdptr, 1, XOSD_string, "");
			}
#endif /* HAVE_LIBXOSD */
		} /* }}} */

		/* sleep for polltime */
		usleep(cfg.polltime);
	}

	_exit(0); /* never reached */
}/*}}}*/

int get_state(t_tpb *tpbs)/*{{{*/
{
	int  fdsc;
	char buffer[114];

	/* open nvram for reading */
	/* must use open/close because seek is not supported by nvram */
	if((fdsc=open(NVRAMDEV, O_RDONLY|O_NONBLOCK)) == -1) {
		perror("Unable to open device");
		return(-1);
	}

	/* read nvram */
	if(read(fdsc,buffer,sizeof(buffer)) != sizeof(buffer)) {
		perror("Unable to read from device");
		return(-1);
	}

	close(fdsc);

	tpbs->tpb=(buffer[0x57] & 0x08) >> 3;
	tpbs->kt=(buffer[0x58] & 0x10) >> 4;
	tpbs->ht=(buffer[0x58] & 0x01);
	tpbs->ds=(buffer[0x59] & 0x03);
	tpbs->bl=(buffer[0x5E] & 0x07);
	tpbs->bt=(buffer[0x5E] & 0x20) >> 5;
	tpbs->vl=(buffer[0x60] & 0x0f);
	tpbs->vt=(buffer[0x60] & 0x80) >> 7;
	tpbs->mt=(buffer[0x60] & 0x40) >> 6;
	tpbs->ac=(buffer[0x57] & 0x10) >> 4;
	tpbs->pma=(buffer[0x39] & 0x07);
	tpbs->pmb=(buffer[0x39] & 0x38) >> 3;

	return(0);
}/*}}}*/

int fork_app(char * cmd) { /* {{{ */
	//char *args[4];
	//args[0] = "sh";
	//args[1] = "-c";
	//args[2] = cmd;
	//args[3] = 0;

	switch(fork()) {
		case -1:
			return(-1);
			break;
		case 0:
			{
				setsid(); /* children should not be killed if tpb ends */
				//execv("/bin/sh", args);
				system(cmd);
				_exit(0);
				return(127);
			}
			break;
		default:
			return(0);
			break;
	}
	return(-1); /* never reached */
} /* }}} */

/* vim:set sw=2:set ts=2: */
/* vim600:set fen:set fdm=marker:set fdl=0: */
